/*
**  WASABI-Hot! version 1.2c    (HID sub-unit)
**
**
**      -- copyright (c) 2001-2004 by Philips Japan, Ltd. -- All rights reserved --
**
**
**      ** This code has been made to check/learn                          ** 
**      **                             the ISP1362/ISP1363 functionalities **
**      ** Release 06-Aug-2004                                             **
**
**      OKANO, Akifumi
**      
**		Application Laboratory, Mobile and Connectivity
**      Semiconductors Div, Philips Japan Ltd.
**      akifumi.okano@philips.com
**      +81-3-3740-4668 
*/

/*
**	HID-Keyboard/Mouse handling framework has been written by Hisashige Harashima
**		04-Mar-2002
*/


/****************************************************************************/
/*	includes																*/
/****************************************************************************/

#include		<stdio.h>
#include		<stdlib.h>
#include		<alloc.h>

#include		"_hc_core/atl_mix.h"
#include		"_hc_core/dev_ep.h"
#include		"_hc_core/port_ctl.h"
#include		"_hc_core/transfer.h"

#include		"_hc_core/dev_ep.h"		// modification by hara
#include		"_hc_cls/cls_hndl.h"	// modification by hara

#include		"class_dr/hid/hid.h"
#include		"class_dr/audio/sing.h"

#include		"ui.h"
#include		"general.h"
#include		"beep.h"


/****************************************************************************/
/*	constants																*/
/****************************************************************************/

#define			KEYBOARD_BUFFER_DEPTH		16


/****************************************************************************/
/*	global vars																*/
/****************************************************************************/

int				g_keyboard_buffer[ KEYBOARD_BUFFER_DEPTH ];
unsigned char	g_keyboard_buffer_ptr_in	= 0;
unsigned char	g_keyboard_buffer_ptr_out	= 0;

hid_cls_interface	hid_class_interface		= { 0, 0, NULL, NULL, NULL };

static char		mojitable[256];																	//Modification by hara

mouse_status	mouse						= { 2, 2, 0 };
mouse_status	p_mouse						= { 0, 0, 0 };

/****************************************************************************/
/*	function prototypes														*/
/****************************************************************************/

device_instance		*create_hid_class_instance( device_instance *dvi_ptr );
void				dispose_hid_class_instance( device_instance *dvi_ptr );
void				hid_callback_keyboard( endpoint_info_ptr epi_ptr );						//Modification by hara
void				hid_callback_mouse( endpoint_info_ptr epi_ptr );						//Modification by hara

void				display_mouse_status( void );

char				key_code_convert_a( char code );										//Modification by hara
char				key_code_convert_A( char code );										//Modification by hara	

void				hid_usb_key_buffer_push( int buffer );
unsigned char		keyboard_buffer_pointer_increment( unsigned char p );

void				moji_init(void);														//Modification by hara
unsigned long		is_new_key_code( unsigned char key_code, unsigned char *previous_key_codes );	//Modification by hara
void				copy_key_codes( unsigned char *target, unsigned char *source );			//Modification by hara
void 				hid_keyboard_main( void );

/****************************************************************************/
/*	function definitions													*/
/****************************************************************************/

unsigned short  	hid_cls_interface_install( hid_cls_interface *hci )
{
	if ( hid_class_interface.idVendor )				//	if already installed
		return ( hid_class_interface.idVendor );	//	do nothing
	
	hid_class_interface		= *hci;
	
	return ( 0 );
}



unsigned short hid_init_commands( device_instance *dvi_ptr )
{
	hid_instance	*hid_i_ptr;
	
	char if_protocol_code;		//Modification by hara.
	
//	mprintf( YELLOW, CONTINUE, "HID connected!\r\n" );

	if ( devep_regist_config_descriptors( dvi_ptr, 1, 0 ) )
		return ( error );

	if ( NULL == create_hid_class_instance( dvi_ptr ) )
		mprintf( LIGHTRED, ABORT, "error @ create_hid_class_instance()\r\n" );


	//	Bus-power turn-on for all ports on the hib.

	hid_i_ptr			= (hid_instance *)hid_instance_ptr( dvi_ptr );


	//	Set interrupt pipe
	
	if ( (NO_OPEN_ATL_TRANSFER_SLOT == (hid_i_ptr->tr_index	= atlmix_get_open_transfer_index( INTERRUPT_TRANSFER ))) )
		mprintf( LIGHTRED, ABORT, "error @ transfer_control_transaction, transfer list full.\r\n" );

	if ( hid_class_interface.idVendor == (dvi_ptr->dev_descriptor).idVendor )
	{
		if ( hid_class_interface.idProduct == (dvi_ptr->dev_descriptor).idProduct )
		{
			if ( 0 != (*(hid_class_interface.hid_initialization_function))( dvi_ptr ) )
				return ( 1 );
		}
	}

// Modification by hara  from here.
		
	if_protocol_code = devep_get_interface_protocol_ID(dvi_ptr);
	
	if ( if_protocol_code == HID_INTERFACE_KEYBOARD)
	{
		hid_i_ptr->tr_instance_ptr		= atlmix_set_transfer( INTERRUPT_TRANSFER, hid_i_ptr->polling_rate, hid_i_ptr->tr_index, (unsigned char *)(&(hid_i_ptr->ep1_data)), hid_i_ptr->max_packet_size, IN, (dvi_ptr->epi_ptr)[ EpIN ][ 1 ], hid_callback_keyboard );

//		mprintf( YELLOW, CONTINUE, "***** Keyboard connected!*****\r\n" );
		moji_init();			//Modification by hara.
	}
	else if ( if_protocol_code == HID_INTERFACE_MOUSE)
	{
		hid_i_ptr->tr_instance_ptr		= atlmix_set_transfer( INTERRUPT_TRANSFER,  hid_i_ptr->polling_rate, hid_i_ptr->tr_index, (unsigned char *)(&(hid_i_ptr->ep1_data)), hid_i_ptr->max_packet_size, IN, (dvi_ptr->epi_ptr)[ EpIN ][ 1 ], hid_callback_mouse );
//		mprintf( YELLOW, CONTINUE, "***** Mouse connected! *****\r\n" );	
	}


// Modification by hara  to here.

//	mprintf( WHITE, CONTINUE, "hid_init_commands %p  %d \r\n", hid_i_ptr, hid_i_ptr->tr_index );


	return ( 0 );
}


device_instance *create_hid_class_instance( device_instance *dvi_ptr )
{
	//	This instance will be disposed in function "devep_dispose_device()"

	hid_instance	*hid_i_ptr;

	if ( NULL == (hid_i_ptr	= (hid_instance *)malloc( 4 + sizeof( hid_instance ) )) )
		mprintf( LIGHTRED, ABORT, "error @ malloc in create_hid_class_instance()\r\n" );

	//	Check endpoint characteristics
	
	if ( (dvi_ptr->conf_descriptor).bNumInterfaces != 1 )
		mprintf( LIGHTRED, ABORT, "error @ create_hid_class_instance, un-supported hid. (dvi_ptr->conf_descriptor).bNumInterfaces is not 1.\r\n" );
	
	if ( ((dvi_ptr->epi_ptr[ EpIN ][ 1 ])->endpoint_descriptor).bEndpointAddress != 0x81 )
		mprintf( LIGHTRED, ABORT, "error @ create_hid_class_instance, un-supported hid. ((dvi_ptr->epi_ptr[ EpIN ][ 1 ])->endpoint_descriptor).bEndpointAddress is not 0x81.\r\n" );
	
	hid_i_ptr->max_packet_size	= ((dvi_ptr->epi_ptr[ EpIN ][ 1 ])->endpoint_descriptor).wMaxPacketSize;
	hid_i_ptr->polling_rate		= ((dvi_ptr->epi_ptr[ EpIN ][ 1 ])->endpoint_descriptor).bInterval;

	dvi_ptr->class_instance_ptr		= hid_i_ptr;

	hid_i_ptr->tr_index				= 0xFF;
	hid_i_ptr->tr_instance_ptr		= NULL;

	return ( dvi_ptr );
}



unsigned short hid_dispose_process( device_instance *dvi_ptr )
{
	hid_instance		*hid_i_ptr;
	device_instance		*downstream_device_ptr;
	int					i;

	if ( hid_class_interface.idVendor == (dvi_ptr->dev_descriptor).idVendor )
		if ( hid_class_interface.idProduct == (dvi_ptr->dev_descriptor).idProduct )
			(*(hid_class_interface.hid_dispose_function))( dvi_ptr );
		
	/*
	**	Stop interrupt transfer to the HID
	*/
	
	hid_i_ptr				= (hid_instance *)hid_instance_ptr( dvi_ptr );

	atlmix_free_transfer_index( INTERRUPT_TRANSFER, hid_i_ptr->tr_index );

	return ( 0 );
}

/*********************************************************
**
**  Keyboard handling.
**
**********************************************************/



void hid_callback_keyboard( endpoint_info_ptr epi_ptr )
{
//
//	KEYBOARD	KEYBOARD	KEYBOARD	KEYBOARD	KEYBOARD	KEYBOARD	KEYBOARD
//

	unsigned long	status;	
	device_instance 		*dvi_ptr;
	static unsigned char	previous_key_codes[ 8 ]	= { 0, 0, 0, 0, 0, 0, 0, 0 };
	unsigned char			key_codes[ 8 ];
	unsigned char			*data_ptr;
	unsigned char			i;
	char					moji		= 0;
	
	if ( atlmix_is_fatal_error( (transfer_instance *)(epi_ptr->transfer_instance) ) )
	{
		//		Device may be detached
		atlmix_free_transfer( (transfer_instance *)(epi_ptr->transfer_instance) );
		return;
	}

	dvi_ptr	= devep_find_device( epi_ptr->dev_addr );

	status	= *((unsigned long *)(((transfer_instance *)(epi_ptr->transfer_instance))->data_ptr_base));
	status = (status & 0x000000FF);
	
	copy_key_codes( key_codes, ((transfer_instance *)(epi_ptr->transfer_instance))->data_ptr_base );
	atlmix_re_enable_interrupt_transfer( epi_ptr->transfer_instance );

	if ( hid_class_interface.idVendor == (dvi_ptr->dev_descriptor).idVendor )
	{
		if ( hid_class_interface.idProduct == (dvi_ptr->dev_descriptor).idProduct )
		{
			(*(hid_class_interface.hid_callback_function))( dvi_ptr, *((unsigned long *)key_codes) );
			return;
		}
	}

	for ( i = 2; i < 8; i++ )
	{
		if ( status == 0x00) 
			moji	= key_code_convert_a( is_new_key_code( key_codes[ i ], previous_key_codes ) );
		
		else if ( status == 0x02)		//Shift right
			moji	= key_code_convert_A( is_new_key_code( key_codes[ i ], previous_key_codes ) );
		
		else if ( status == 0x20)		//Shift left
			moji	= key_code_convert_A( is_new_key_code( key_codes[ i ], previous_key_codes ) );
		
		if ( moji )
			hid_usb_key_buffer_push( moji & 0xFF );
	}
	
	copy_key_codes( previous_key_codes, key_codes );
}


void copy_key_codes( unsigned char *target, unsigned char *source )
{
	unsigned char	i;
	
	for ( i = 0; i < 8; i++ )
		*target++	= *source++;
}


unsigned long is_new_key_code( unsigned char key_code, unsigned char *previous_key_codes )
{
	unsigned char			i;
	
	for ( i = 2; i < 8; i++ )
		if ( key_code == previous_key_codes[ i ] )
			return ( 0 );
	
	return ( key_code );

}



/*********************************************************
**
**  Convert "charactor" from USB to ASCII from mojitable[i]
**
**********************************************************/


char key_code_convert_a( char code )
{
	
	if( (code >= 0x04) && (code <= 0x38) )			
		return (mojitable[code]);
			
		else
			return (0);	
}

/*********************************************************
**
**  Convert "charactor with Shift key" from USB to ASCII from mojitable[i]
**
**********************************************************/


char key_code_convert_A( char code )
{	
		
	if( (code >= 0x04) && (code <= 0x38) )			
		return (mojitable[code + 53]);	
	else
		return (0);	

}



/*************************************************
**
**  Mouse hanbdling
**
*************************************************/


void hid_callback_mouse( endpoint_info_ptr epi_ptr )
{
//
//	MOUSE	MOUSE	MOUSE	MOUSE	MOUSE	MOUSE	MOUSE	MOUSE	MOUSE	MOUSE	MOUSE	MOUSE
//

	device_instance *dvi_ptr;
	unsigned long	status;
	short			x;
	short			y;
	unsigned char	button;
	
//	mprintf( WHITE, CONTINUE, "hid_callback_mouse          \r\n" );


	if ( atlmix_is_fatal_error( (transfer_instance *)(epi_ptr->transfer_instance) ) )
	{
		//		Device may be detached
		atlmix_free_transfer( (transfer_instance *)(epi_ptr->transfer_instance) );
		return;
	}



	dvi_ptr	= devep_find_device( epi_ptr->dev_addr );

	status	= *((unsigned long *)(((transfer_instance *)(epi_ptr->transfer_instance))->data_ptr_base));
	
	atlmix_re_enable_interrupt_transfer( epi_ptr->transfer_instance );

	if ( hid_class_interface.idVendor == (dvi_ptr->dev_descriptor).idVendor )
	{
		if ( hid_class_interface.idProduct == (dvi_ptr->dev_descriptor).idProduct )
		{
			(*(hid_class_interface.hid_callback_function))( dvi_ptr, status );
			return;
		}
		
	}

#if 0
	x	= mouse.x + ((signed char)((status & 0x0000FF00) >>  8) >> 1);
	y	= mouse.y + ((signed char)((status & 0x00FF0000) >> 16) >> 1);
#else

	x	= (signed char)((status & 0x0000FF00) >>  8);
	y	= (signed char)((status & 0x00FF0000) >> 16);
	
	x	= (x < 0) ? -1 : x;
	x	= (0 < x) ?  1 : x;
	y	= (y < 0) ? -1 : y;
	y	= (0 < y) ?  1 : y;
	
	x	+= mouse.x;
	y	+= mouse.y;
	
#endif
	
	x	= (  x < 1 ) ?  1 : x;
	x	= ( 78 < x ) ? 78 : x;
	y	= (  y < 1 ) ?  1 : y;
	y	= ( 14 < y ) ? 14 : y;
	
	button	= (signed char)(status & 0x000000FF);
	
	p_mouse			= mouse;

	mouse.x			= x;
	mouse.y			= y;
	mouse.button	= button;
	
	if ( mouse.button & p_mouse.button & 0x02 )
		audio_volume_set_delta( p_mouse.x - mouse.x );
	else
		gene_install_asynchronous_periodic_process( 4, display_mouse_status );
}


#if 0
void display_mouse_status( void )
{
	gene_install_asynchronous_periodic_process( 4, NULL );

	mprintf( WHITE, CONTINUE, "    mouse event [x = %2d, y = %2d] ", mouse.x, mouse.y );
	mprintf( (mouse.button & 0x01) ? WHITE : DARKGRAY, CONTINUE, "[Left] " );
	mprintf( (mouse.button & 0x02) ? WHITE : DARKGRAY, CONTINUE, "[Right]\r\n" );
}
#else
void display_mouse_status( void )
{
	unsigned char			color;
	unsigned char			p_color;
	unsigned char			message_on;
	int						x;
	int						y;
	static int				px;
	static int				py;
	
	gene_install_asynchronous_periodic_process( 4, NULL );

	x	= wherex();
	y	= wherey();
	
	if ( (p_mouse.button | mouse.button) & 0x02 )
	{
		message_on		= mouse.button & 0x02;
		
		if ( message_on )
		{
			px		= ((78 - 17) < mouse.x) ? (78 - 17) : mouse.x;
			py		= mouse.y;
		}
		
		gotoxy( px, py );

		if ( is_audio_exist() )
			mprintf( message_on ? LIGHTRED : DARKGRAY, CONTINUE, "[Volume control!]" );
		else
			mprintf( message_on ? LIGHTRED : DARKGRAY, CONTINUE, "[No Audio device!]" );

		gotoxy( x, y );
		return;
	}

	if ( mouse.x )
	{
		switch ( p_mouse.button )
		{
			case 0x01 : 
				color	= LIGHTRED;
				break;
			case 0x03 : 
				color	= YELLOW;
				break;
			default : 
				color	= DARKGRAY;
				break;
		}

		gotoxy( p_mouse.x, p_mouse.y );
		mprintf( color, CONTINUE, "*" );
	}

	switch ( mouse.button )
	{
		case 0x01 : 
			color	= LIGHTRED;
			break;
		case 0x03 : 
			color	= YELLOW;
			break;
		default : 
			color	= WHITE;
			break;
	}

	gotoxy( mouse.x, mouse.y );
//	mprintf( color , CONTINUE, "%c", 219 );
	mprintf( color , CONTINUE, "%c", 16 );
	gotoxy( x, y );
}
#endif


unsigned char hid_usb_key_buffer( void )
{
	return ( g_keyboard_buffer_ptr_out - g_keyboard_buffer_ptr_in );
}


int hid_usb_key_buffer_pop( void )
{
	int		buffer;

	if ( g_keyboard_buffer_ptr_out != g_keyboard_buffer_ptr_in )
	{
		buffer						= g_keyboard_buffer[ g_keyboard_buffer_ptr_out ];
		g_keyboard_buffer_ptr_out	= keyboard_buffer_pointer_increment( g_keyboard_buffer_ptr_out );

		return( buffer );
	}
	
	return( 0 );
}


void hid_usb_key_buffer_push( int buffer )
{
	unsigned char	p;

	p	= keyboard_buffer_pointer_increment( g_keyboard_buffer_ptr_in );

	if ( g_keyboard_buffer_ptr_out != p )
	{
		g_keyboard_buffer[ g_keyboard_buffer_ptr_in ]	= buffer;
		g_keyboard_buffer_ptr_in						= p;
	}
}


unsigned char keyboard_buffer_pointer_increment( unsigned char p )
{
	return ( ((p + 1) < KEYBOARD_BUFFER_DEPTH) ? ++p : 0 );
}


void hid_keyboard_main( void )
{
	char s[80];

	mprintf( LIGHTGREEN, CONTINUE, "\r\n\r\n  KeyboardTest started\r\n\r\n" );
	mprintf( LIGHTGRAY,  CONTINUE, "    Keyboard test!! Please input from your USB keyboard. \r\n    Hit " );
	mprintf( LIGHTRED,   CONTINUE, "[Return]" );
	mprintf( LIGHTGRAY,  CONTINUE, " to exit from this test routine.\r\n\r\n" );
	ui_get_string_from_console( s, 80, WHITE, "    >> ", YELLOW, 0, "Hit keys to test.", NULL );
	mprintf( YELLOW, CONTINUE, "\r\n" );
	ui_soft_blink(s, 150, 2);
	mprintf( YELLOW, CONTINUE, "\r\n\r\n\r\n" );
}

/*************************************************
**
**  Write the each charactor on mojitable[i]
**
*************************************************/



void moji_init( void )				
{
	int i;
	
	for(i = 4; i<30; i++)
		{
		mojitable[i] = 93 + i; 		//alpha
		}  
	
	for(i = 30; i<39; i++)			//number1-9
		{
		mojitable[i] =  19 + i;
		}  
	
	mojitable[39] = 48;				//number0*

	mojitable[40] = 13;				//enter
	mojitable[41] = 27;				//escape	(ESP)
	mojitable[42] = 8;				//Backspace (BS)
	mojitable[43] = 9;				//Tab
	mojitable[44] = 32;				//Keyboard space
	mojitable[45] = 45;				// -
	mojitable[46] = 61;				// =
	mojitable[47] = 91;				// [
	mojitable[48] = 93;				// ]
	mojitable[49] = 92;				// \
	//mojitable[50] = 96;			// `	 
	mojitable[51] = 59;				// ;
	mojitable[52] = 39;				// '
	//mojitable[53] = ;				//
	mojitable[54] = 44;				// ,
	mojitable[55] = 46;				// .
	mojitable[56] = 47;				// /
	
	//Shift
	
	for(i = 57; i<83; i++)
		{
		mojitable[i] = 8 + i; 		//ALPHA
		}
	
	mojitable[83] = 33;				// !
	mojitable[84] = 64;				// @
	mojitable[85] = 35;				// #
	mojitable[86] = 36;				// $
	mojitable[87] = 37;				// %
	mojitable[88] = 94;				// ^
	mojitable[89] = 38;				// &
	mojitable[90] = 42;				// *
	mojitable[91] = 40;				// (
	mojitable[92] = 41;				// )
	//mojitable[93] = ;			
	//mojitable[94] = ;			
	//mojitable[95] = ;			
	//mojitable[96] = ;			
	//mojitable[97] = ;			
	mojitable[98] = 95;				// _
	mojitable[99] = 43;				// +
	mojitable[100] = 123;			// {
	mojitable[101] = 125;			// }
	mojitable[102] = 124;			// |
	//mojitable[103] = 126;			// ~
	mojitable[104] = 58;			// :
	mojitable[105] = 34;			// "
	//mojitable[106] = ;			// 
	mojitable[107] = 60;			// <
	mojitable[108] = 62;			// >
	mojitable[109] = 63;			// ?

}		 



/*

Moji_table (Hex)

Low		|						High Nibble
Nibble	|	0		1		2		3		4		5		6		7		
________|_______________________________________________________________________
		|
	0	|			m		3		]		H		X									
		|
	1	|			n		4		\		I		Y									
		|
	2	|			o		5				J		Z		_							
		|
	3	|			p		6		;		K		!		+							
		|
	4	|	a		q		7		'		L		@		{							
		|
	5	|	b		r		8				M		#		}							
		|
	6	|	c		s		9		,		N		$		|							
		|
	7	|	d		t		0		.		O		%									
		|
	8	|	e		u		Enter	/		P		^		:								
		|
	9	|	f		v		ESC		A		Q		&		"							
		|
	A	|	g		w		BS		B		R		*									
		|
	B	|	h		x		Tab		C		S		(		<							
		|
	C	|	i		y		Space	D		T		)		>								
		|
	D	|	j		z		-		E		U				?
		|
	E	|	k		1		=		F		V											
		|
	F	|	l		2		[		G		W											
	
*/












