/*
**  WASABI-Hot! version 1.2c    (for Griffin PowerMate)
**
**
**      -- copyright (c) 2001-2004 by Philips Japan, Ltd. -- All rights reserved --
**
**
**      ** This code has been made to check/learn                          ** 
**      **                             the ISP1362/ISP1363 functionalities **
**      ** Release 06-Aug-2004                                             **
**
**      OKANO, Akifumi
**      
**		Application Laboratory, Mobile and Connectivity
**      Semiconductors Div, Philips Japan Ltd.
**      akifumi.okano@philips.com
**      +81-3-3740-4668 
*/


/****************************************************************************/
/*	includes																*/
/****************************************************************************/

#include	"_hc_core/dev_ep.h"
#include	"class_dr/hid/hid.h"
#include	"class_dr/audio/sing.h"

#include	"griffin.h"
#include	"ui.h"
#include	"beep.h"


/****************************************************************************/
/*	constants																*/
/****************************************************************************/

#define				GRIFFIN_VENDER_SPECIFIC_COMMAND		0x4101

#define				COMMAND_LIGHT_BLINKING_SPEED		4
#define				COMMAND_LIGHT_BLINKING_START_STOP	3

#define				VENDER_ID_FOR_GRIFFIN_TECHNOLOGY	0x077D;
#define				PRODUCT_ID_FOR_POWER_MATE			0x0410;


/****************************************************************************/
/*	global vars																*/
/****************************************************************************/

unsigned char		griffin_target_address			= 0;
unsigned long		g_previous_state				= 0xFF;
unsigned char		g_mute_state;

/****************************************************************************/
/*	function prototypes														*/
/****************************************************************************/

unsigned char		griffin_get_parameter( device_instance *dvi_ptr, unsigned long value );
unsigned char		griffin_initialize_device( device_instance *dvi_ptr );
unsigned char		griffin_dispose_device( device_instance *dvi_ptr );
device_instance		*griffin_get_device( void );
void				change_blink_speed_by_mute_state( void );


/****************************************************************************/
/*	function definitions													*/
/****************************************************************************/

void griffin_initialize( void )
{
	hid_cls_interface	hci;
	
	hci.idVendor						= VENDER_ID_FOR_GRIFFIN_TECHNOLOGY;
	hci.idProduct						= PRODUCT_ID_FOR_POWER_MATE;
	hci.hid_initialization_function		= griffin_initialize_device;
	hci.hid_dispose_function			= griffin_dispose_device;
	hci.hid_callback_function			= griffin_get_parameter;
	
	if ( hid_cls_interface_install( &hci ) )
		mprintf( LIGHTRED, CONTINUE, "error @ hid_cls_interface_install in griffin_initialize()\r\n" );
}


unsigned char griffin_get_parameter( device_instance *dummy_for_func_compatibility, unsigned long value )
{
	char					rotate;
	unsigned char			mute;

	dummy_for_func_compatibility	= dummy_for_func_compatibility;	//	This line for erase warning

	if ( g_previous_state == 0xFF )
	{
		g_previous_state	= 0;
		return ( 0 );
	}

	if ( (value & 0x0000FF00) && !(value & 0x1) )
	{
		rotate		= ~(char)(value >> 8) + 1;

		if ( rotate )
			audio_volume_set_delta( rotate );
	}

	if ( ~value & g_previous_state & 0x00000001 )
	{
		g_mute_state	= audio_volume_mute_toggle();
		gene_install_asynchronous_periodic_process( 4, change_blink_speed_by_mute_state );
	}
	g_previous_state	 = value;
	
	return ( 0 );
}


unsigned char griffin_initialize_device( device_instance *dvi_ptr )
{
	unsigned short		rx_size;
	unsigned char		dummy;

	griffin_target_address	= dvi_ptr->address;

	rx_size		= 0;
	
//	mprintf( LIGHTRED, CONTINUE, "Griffin taget initializing...\r\n" );

	if ( devep_std_request( &dummy, dvi_ptr, GRIFFIN_VENDER_SPECIFIC_COMMAND, COMMAND_LIGHT_BLINKING_SPEED, 0x2020, &rx_size ) )
	{
		mprintf( LIGHTRED, CONTINUE, "Griffin taget has error, disabled\r\n" );
		griffin_target_address	= 0;
		
		return ( 1 );
	}	

	if ( devep_std_request( &dummy, dvi_ptr, GRIFFIN_VENDER_SPECIFIC_COMMAND, COMMAND_LIGHT_BLINKING_START_STOP, 1, &rx_size ) )
	{
		mprintf( LIGHTRED, CONTINUE, "Griffin taget has error, disabled\r\n" );
		griffin_target_address	= 0;

		return ( 1 );
	}

	return ( 0 );
}


unsigned char griffin_dispose_device( device_instance *dvi_ptr )
{
	if ( griffin_target_address	!= dvi_ptr->address )
		mprintf( LIGHTRED, CONTINUE, "error @ griffin_dispose_device()\r\n" );
	
	griffin_target_address			= 0;
	g_previous_state				= 0xFF;

	return ( 0 );
}


void griffin_set_parameter( unsigned short command, unsigned short value )
{
	device_instance		*dvi_ptr;
	unsigned short		rx_size;
	unsigned long		dummy;

	if ( !griffin_target_address )
		return;
	
	dvi_ptr		= griffin_get_device();

	rx_size		= 0;
	
	if ( devep_std_request( (unsigned char *)(&dummy), dvi_ptr, GRIFFIN_VENDER_SPECIFIC_COMMAND, command, value, &rx_size ) )
	{
		mprintf( LIGHTRED, CONTINUE, "Griffin taget has error, disabled\r\n" );
		griffin_target_address	= 0;
	}	
}


device_instance *griffin_get_device( void )
{
	return ( devep_find_device( griffin_target_address ) );
}


void change_blink_speed_by_mute_state( void )
{
	gene_install_asynchronous_periodic_process( 4, NULL );
	griffin_set_parameter( COMMAND_LIGHT_BLINKING_SPEED, g_mute_state ? 0x0E02 : 0x2020 );
}




